@@ -18,7 +18,7 @@
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- * (C) Copyright 2004 - 2015 Red Hat, Inc.
+ * (C) Copyright 2004 - 2018 Red Hat, Inc.
  */
 
 #include "nm-default.h"
@@ -154,7 +154,6 @@ eui_finish (const char *vpn_name,
             const char *existing_group_password)
 {
 	GKeyFile *keyfile;
-	char *title;
 	gboolean show;
 
 	keyfile = g_key_file_new ();
@@ -162,9 +161,7 @@ eui_finish (const char *vpn_name,
 	g_key_file_set_integer (keyfile, UI_KEYFILE_GROUP, "Version", 2);
 	g_key_file_set_string (keyfile, UI_KEYFILE_GROUP, "Description", prompt);
 
-	title = g_strdup_printf (_("Authenticate VPN %s"), vpn_name);
-	g_key_file_set_string (keyfile, UI_KEYFILE_GROUP, "Title", title);
-	g_free (title);
+	g_key_file_set_string (keyfile, UI_KEYFILE_GROUP, "Title", _("Authenticate VPN"));
 
 	/* If we have an existing password, or we need the user to give us one,
 	 * then tell the external UI about the password.  An entry for the password
@@ -179,7 +176,7 @@ eui_finish (const char *vpn_name,
 		keyfile_add_entry_info (keyfile,
 		                        NM_LIBRESWAN_KEY_XAUTH_PASSWORD,
 		                        existing_password ? existing_password : "",
-		                        _("Password:"),
+		                        _("Password"),
 		                        TRUE,
 		                        show && allow_interaction);
 	}
@@ -189,7 +186,7 @@ eui_finish (const char *vpn_name,
 		keyfile_add_entry_info (keyfile,
 		                        NM_LIBRESWAN_KEY_PSK_VALUE,
 		                        existing_group_password ? existing_group_password : "",
-		                        _("Group Password:"),
+		                        _("Group Password"),
 		                        TRUE,
 		                        show && allow_interaction);
 	}
@@ -225,6 +222,8 @@ std_ask_user (const char *vpn_name,
 	g_return_val_if_fail (out_new_password != NULL, FALSE);
 	g_return_val_if_fail (out_new_group_password != NULL, FALSE);
 
+	gtk_init (NULL, NULL);
+
 	dialog = NMA_VPN_PASSWORD_DIALOG (nma_vpn_password_dialog_new (_("Authenticate VPN"), prompt, NULL));
 
 	/* pre-fill dialog with existing passwords */
@@ -439,11 +438,11 @@ main (int argc, char *argv[])
 
 	bindtextdomain (GETTEXT_PACKAGE, NULL);
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
-	gtk_init (&argc, &argv);
 	textdomain (GETTEXT_PACKAGE);
 
 	context = g_option_context_new ("- IPsec auth dialog");
 	g_option_context_add_main_entries (context, entries, GETTEXT_PACKAGE);
+	g_option_context_add_group (context, gtk_get_option_group (FALSE));
 
 	if (!g_option_context_parse (context, &argc, &argv, &error)) {
 		fprintf (stderr, "Error parsing options: %s\n", error->message);
